# -*- coding: utf-8; -*-
################################################################################
#
#  Rattail -- Retail Software Framework
#  Copyright © 2010-2020 Lance Edgar
#
#  This file is part of Rattail.
#
#  Rattail is free software: you can redistribute it and/or modify it under the
#  terms of the GNU General Public License as published by the Free Software
#  Foundation, either version 3 of the License, or (at your option) any later
#  version.
#
#  Rattail is distributed in the hope that it will be useful, but WITHOUT ANY
#  WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
#  FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
#  details.
#
#  You should have received a copy of the GNU General Public License along with
#  Rattail.  If not, see <http://www.gnu.org/licenses/>.
#
################################################################################
"""
CORE-POS model importers (webservices API)
"""

from corepos.api import CoreWebAPI

from rattail import importing


class ToCoreAPI(importing.Importer):
    """
    Base class for all CORE "operational" model importers, which use the API.
    """
    caches_local_data = True

    def setup(self):
        self.establish_api()

    def establish_api(self):
        url = self.config.require('corepos.api', 'url')
        self.api = CoreWebAPI(url)

    def ensure_fields(self, data):
        """
        Ensure each of our supported fields are included in the data.  This is
        to handle cases where the API does not return all fields, e.g. when
        some of them are empty.
        """
        for field in self.fields:
            if field not in data:
                data[field] = None

    def fix_empties(self, data, fields):
        """
        Fix "empty" values for the given set of fields.  This just uses an
        empty string instead of ``None`` for each, to add some consistency
        where the API might lack it.

        Main example so far, is the Vendor API, which may not return some
        fields at all (and so our value is ``None``) in some cases, but in
        other cases it *will* return a value, default of which is the empty
        string.  So we want to "pretend" that we get an empty string back even
        when we actually get ``None`` from it.
        """
        for field in fields:
            if data[field] is None:
                data[field] = ''


class VendorImporter(ToCoreAPI):
    """
    Vendor model importer for CORE-POS
    """
    model_name = 'Vendor'
    key = 'vendorID'
    supported_fields = [
        'vendorID',
        'vendorName',
        'vendorAbbreviation',
        'shippingMarkup',
        'discountRate',
        'phone',
        'fax',
        'email',
        'website',
        'address',
        'city',
        'state',
        'zip',
        'notes',
        'localOriginID',
        'inactive',
        'orderMinimum',
        'halfCases',
    ]
    # TODO: this importer is in a bit of an experimental state at the moment.
    # we only allow "update" b/c it will use the API instead of direct DB
    allow_create = False
    allow_delete = False

    def get_local_objects(self, host_data=None):
        return self.api.get_vendors()

    def get_single_local_object(self, key):
        assert len(self.key) == 1
        assert self.key[0] == 'vendorID'
        return self.api.get_vendor(key[0])

    def normalize_local_object(self, vendor):
        data = dict(vendor)

        # make sure all fields are present
        self.ensure_fields(data)

        # fix some "empty" values
        self.fix_empties(data, ['phone', 'fax', 'email'])

        # convert some values to native type
        data['discountRate'] = float(data['discountRate'])

        return data

    def update_object(self, vendor, data, local_data=None):
        """
        Push an update for the vendor, via the CORE API.
        """
        if self.dry_run:
            return data

        vendorID = data.pop('vendorID')
        self.api.set_vendor(vendorID, **data)
        return data
